package se2030.parserdemo;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import se2030.parser.AbstractParserEventHandler;

/*
 * This class provides a sample concrete implementation of the AbstractParserEventHandler,
 * which is an abstract class that provides default implementations of the
 * event handling methods that are called by the Parser.
 * 
 * It provides implementation to the abstract methods of AbstractParserEventHandler.
 * In this handler, these implemented methods simply announce (via System.out.println)
 * what XML elements have been "discovered" by the Parser.
 *  
 */
public class ElementAnnouncer extends AbstractParserEventHandler {

	@Override
	// This ContentHandler method is called whenever the Parser encounters a new XML element tag
	// which may or may not have Attributes. 
	public void startElement(String uri, String localName, String qName, Attributes atts) throws SAXException {
		int line = locator.getLineNumber(); // current line being parsed
		int column = locator.getColumnNumber(); // current column being parsed
		log("startElement found", "<"+localName+"> at line " + line + ", col " + column );
		if( atts.getLength()> 0 ) { // check for element attributes; print them only if they exist.
			String attributes = "";
			for( int index=0; index<atts.getLength(); index++) {
				attributes += " "+ atts.getLocalName(index) + "=" + atts.getValue(index);
			}
			log("<"+localName+"> attributes", attributes);
		}
	}

	@Override
	// This ContentHandler method is called whenever the Parser encounters an XML end tag
	public void endElement(String uri, String localName, String qName)
			throws SAXException {
		int line = locator.getLineNumber(); // current line being parsed
		int column = locator.getColumnNumber(); // current column being parsed
		log("endElement found", "<"+localName+"> at line " + line + ", col " + column );
	}

	@Override
	// This ContentHandler method is called whenever the Parser encounters text between tags.
	public void characters(char[] ch, int start, int length) throws SAXException {
		String s = new String(ch); // convert char[] to String
		s = s.substring(start, start+length);
		s = s.trim(); // remove leading and trailing whitespace

		if( !s.isEmpty() ) { // ignore whitespace
			int line = locator.getLineNumber(); // current line being parsed
			int column = locator.getColumnNumber(); // current column being parsed
			log("characters found", "'"+s + "' at line " + line + ", col " + column );
		}
	}
	
	@Override
	//This ContentHandler method is called when the Parser reaches the end of the
	// XML document. 
	public void endDocument() throws SAXException {
		log("end of document found", "parsing complete" ); // debug logger
	}

}